@@ -13,9 +13,21 @@ except ImportError:
     IS_OPENCL_AVAILABLE = False
 
 
+@njit
+def update_repulsion_choice(G_m, max_weight, weights, n, used, node):
+    # Select node
+    used[node] = True
+
+    # Repulsion: penalize neighbors
+    for nbr in range(n):
+        if used[nbr]:
+            continue
+        weights[nbr] *= max(2e-7, 1 - G_m[node, nbr] / max_weight)
+
+
 # Written by Elara (OpenAI custom GPT) and improved by Dan Strano
 @njit
-def local_repulsion_choice(G_m, max_weight, weights, n, m):
+def local_repulsion_choice(G_m, max_weight, weights, n, m, shot):
     """
     Pick m nodes out of n with repulsion bias:
     - High-degree nodes are already less likely
@@ -27,7 +39,10 @@ def local_repulsion_choice(G_m, max_weight, weights, n, m):
     weights = weights.copy()
     used = np.zeros(n, dtype=np.bool_) # False = available, True = used
 
-    for _ in range(m):
+    # Update answer and weights
+    update_repulsion_choice(G_m, max_weight, weights, n, used, shot % n)
+
+    for _ in range(1, m):
         # Count available
         total_w = 0.0
         for i in range(n):
@@ -52,14 +67,8 @@ def local_repulsion_choice(G_m, max_weight, weights, n, m):
             while used[node]:
                 node += 1
 
-        # Select node
-        used[node] = True
-
-        # Repulsion: penalize neighbors
-        for nbr in range(n):
-            if used[nbr]:
-                continue
-            weights[nbr] *= max(2e-7, 1 - G_m[node, nbr] / max_weight)
+        # Update answer and weights
+        update_repulsion_choice(G_m, max_weight, weights, n, used, node)
 
     return used
 
@@ -92,7 +101,7 @@ def sample_for_solution(G_m, shots, thresholds, J_eff):
         m += 1
 
         # Second dimension: permutation within Hamming weight
-        sample = local_repulsion_choice(G_m, max_weight, weights, n, m)
+        sample = local_repulsion_choice(G_m, max_weight, weights, n, m, s)
 
         solutions[s] = sample
         energies[s] = compute_energy(sample, G_m, n)
@@ -126,13 +135,13 @@ def init_J_and_z(G_m):
     return J_eff, degrees
 
 
-def run_sampling_opencl(G_m_np, thresholds_np, shots, n):
+def run_sampling_opencl(G_m_np, thresholds_np, shots, n, is_g_buf_reused):
     ctx = opencl_context.ctx
     queue = opencl_context.queue
     kernel = opencl_context.sample_for_solution_best_bitset_kernel
 
     max_local_size = 64  # tune
-    max_global_size = 65536
+    max_global_size = ((opencl_context.MAX_GPU_PROC_ELEM + max_local_size - 1) // max_local_size) * max_local_size  # corresponds to MAX_PROC_ELEM macro in OpenCL kernel program
     global_size = min(((shots + max_local_size - 1) // max_local_size) * max_local_size, max_global_size)
     local_size = max_local_size
     num_groups = global_size // local_size
@@ -167,6 +176,7 @@ def run_sampling_opencl(G_m_np, thresholds_np, shots, n):
         thresholds_buf,
         np.int32(n),
         np.int32(shots),
+        np.float64(G_m_np.max()),
         rng_buf,
         solutions_buf,
         best_energies_buf,
@@ -195,6 +205,9 @@ def run_sampling_opencl(G_m_np, thresholds_np, shots, n):
         b = u & 31
         best_solution[u] = (best_solution_bits[w] >> b) & 1
 
+    if is_g_buf_reused:
+        opencl_context.G_m_buf = G_m_buf
+
     return best_solution, best_energy
 
 
@@ -226,7 +239,9 @@ def gpu_footer(shots, n_qubits, G_m, J_eff, hamming_prob, nodes):
 def maxcut_tfim(
     G,
     quality=None,
-    shots=None
+    shots=None,
+    is_alt_gpu_sampling = False,
+    is_g_buf_reused = False
 ):
     nodes = None
     n_qubits = 0
@@ -283,7 +298,7 @@ def maxcut_tfim(
     args_buf = cl.Buffer(opencl_context.ctx, mf.READ_ONLY | mf.COPY_HOST_PTR, hostbuf=args)
     J_buf = cl.Buffer(opencl_context.ctx, mf.READ_ONLY | mf.COPY_HOST_PTR, hostbuf=J_eff)
     deg_buf = cl.Buffer(opencl_context.ctx, mf.READ_ONLY | mf.COPY_HOST_PTR, hostbuf=degrees)
-    theta_buf = cl.Buffer(opencl_context.ctx, mf.READ_WRITE, size=(n_qubits * 8))
+    theta_buf = cl.Buffer(opencl_context.ctx, mf.READ_WRITE, size=(n_qubits * np.float32().nbytes))
 
     # Warp size is 32:
     group_size = min(n_qubits, 64)
@@ -325,4 +340,11 @@ def maxcut_tfim(
     deg_buf = None
     theta_buf = None
 
-    return gpu_footer(shots, n_qubits, G_m, J_eff, hamming_prob, nodes)
+    if not is_alt_gpu_sampling:
+        return gpu_footer(shots, n_qubits, G_m, J_eff, hamming_prob, nodes)
+
+    fix_cdf(hamming_prob)
+    best_solution, best_value = run_sampling_opencl(G_m, hamming_prob, shots, n_qubits, is_g_buf_reused)
+    bit_string, l, r = get_cut(best_solution, nodes)
+
+    return bit_string, best_value, (l, r)
