@@ -143,7 +143,8 @@ def probability_by_hamming_weight(J, h, z, theta, t, n_qubits):
 
 
 class OpenCLContext:
-    def __init__(self, a, c, q, i, m, b, s):
+    def __init__(self, p, a, c, q, i, m, b, s, k, l):
+        self.MAX_GPU_PROC_ELEM = p
         self.IS_OPENCL_AVAILABLE = a
         self.ctx = c
         self.queue = q
@@ -151,30 +152,43 @@ class OpenCLContext:
         self.maxcut_hamming_cdf_kernel = m
         self.bootstrap_kernel = b
         self.bootstrap_sparse_kernel = s
+        self.sample_for_solution_best_bitset_kernel = k
+        self.sample_for_solution_best_bitset_sparse_kernel = l
+        self.G_m_buf = None
+        self.G_data_buf = None
+        self.G_rows_buf = None
+        self.G_cols_buf = None
 
 IS_OPENCL_AVAILABLE = True
 ctx = None
 queue = None
+compute_units = None
 init_theta_kernel = None
 maxcut_hamming_cdf_kernel = None
 bootstrap_kernel = None
 bootstrap_sparse_kernel = None
+sample_for_solution_best_bitset_kernel = None
+sample_for_solution_best_bitset_sparse_kernel = None
 try:
     import pyopencl as cl
     
     # Pick a device (GPU if available)
     ctx = cl.create_some_context()
     queue = cl.CommandQueue(ctx)
+    compute_units = int(os.getenv('PYQRACKISING_MAX_GPU_PROC_ELEM', str(ctx.devices[0].get_info(cl.device_info.MAX_COMPUTE_UNITS))))
 
     # Load and build OpenCL kernels
-    kernel_src = open(os.path.dirname(os.path.abspath(__file__)) + "/kernels.cl").read()
+    kernel_src = f"#define MAX_PROC_ELEM {compute_units}\n"
+    kernel_src += open(os.path.dirname(os.path.abspath(__file__)) + "/kernels.cl").read()
     program = cl.Program(ctx, kernel_src).build()
     init_theta_kernel = program.init_theta
     maxcut_hamming_cdf_kernel = program.maxcut_hamming_cdf
     bootstrap_kernel = program.bootstrap
     bootstrap_sparse_kernel = program.bootstrap_sparse
+    sample_for_solution_best_bitset_kernel = program.sample_for_solution_best_bitset
+    sample_for_solution_best_bitset_sparse_kernel = program.sample_for_solution_best_bitset_sparse
 except ImportError:
     IS_OPENCL_AVAILABLE = False
     print("PyOpenCL not installed. (If you have any OpenCL accelerator devices with available ICDs, you might want to optionally install pyopencl.)")
 
-opencl_context = OpenCLContext(IS_OPENCL_AVAILABLE, ctx, queue, init_theta_kernel, maxcut_hamming_cdf_kernel, bootstrap_kernel, bootstrap_sparse_kernel)
+opencl_context = OpenCLContext(compute_units, IS_OPENCL_AVAILABLE, ctx, queue, init_theta_kernel, maxcut_hamming_cdf_kernel, bootstrap_kernel, bootstrap_sparse_kernel, sample_for_solution_best_bitset_kernel, sample_for_solution_best_bitset_sparse_kernel)
